package com.tcl.remotelog.log

import android.text.TextUtils
import android.util.Log
import android.widget.Toast
import com.tcl.remotelog.RemoteLogHelper
import com.tcl.remotelog.model.AddLogcatReq
import com.tcl.remotelog.model.DeviceStatusReq
import com.tcl.remotelog.net.LogHttpHelper
import com.tcl.remotelog.net.RetryWithDelay
import com.tcl.remotelog.utils.CDNHelper
import com.tcl.remotelog.utils.ConvertUtils
import com.tcl.remotelog.utils.LocalLogUtils
import com.tcl.remotelog.utils.RxUtils
import com.tcl.remotelog.utils.ZipUtils
import io.reactivex.Observable
import java.io.BufferedReader
import java.io.File
import java.io.IOException
import java.io.InputStreamReader
import java.util.Locale
import java.util.UUID


object LogcatHelper {
    private var logThread: Thread? = null
    private var currentTaskProcess: Process? = null
    private var logFile: File? = null
    private var currentRandomId = ""
    private var currentPackageName = ""
    private var currentStatus: LogState = LogState.TYPE_INIT


    private fun createRandomId(): String {
        return UUID.randomUUID().toString().replace("-", "")
    }

    /**
     * 上传日志
     */
    private fun uploadLog(key: String, size: Long) {
        LogHttpHelper.service.addLogcat(
            AddLogcatReq(
                key, RemoteLogHelper.getLocalMac(), ConvertUtils.convertBytesToSize(size)
            )
        ).retryWhen(RetryWithDelay()).subscribeOn(RxUtils.getSchedulerIO()).subscribe({}, {
                it.printStackTrace()
            })
    }

    private fun initLogEngine() {
        //打开日志权限
        Runtime.getRuntime().exec("setprop persist.sys.logd.level V")
        //打开adb
        Runtime.getRuntime().exec("setprop persist.sys.usb.config adb")
        Runtime.getRuntime().exec("setprop sys.usb.config adb")
        Runtime.getRuntime().exec("setprop persist.sys.service.adbd.enable 1")
        Thread.sleep(2000)
    }

    @Synchronized
    fun startLogCatch(packageName: String = "", call: (log: String) -> Unit = {}) {
        if (logThread?.isAlive == true) return
        currentStatus = LogState.TYPE_CATCHING
        currentRandomId = createRandomId()
        currentPackageName = packageName
        logFile = File(
            RemoteLogHelper.getContext()?.externalCacheDir,
            "log-$currentRandomId-${if (TextUtils.isEmpty(packageName)) "all" else packageName}"
        )
        logThread = Thread {
            try {
                //删除历史文件
                val cacheDir = RemoteLogHelper.getContext()?.externalCacheDir
                if (cacheDir?.exists() == true) {
                    val listFiles = cacheDir.listFiles()
                    listFiles.sortBy { it.lastModified() }
                    if (listFiles.size > 1) {
                        listFiles.forEachIndexed { index, file ->
                            if (index != listFiles.size - 1) {
                                file.deleteOnExit()
                            }
                        }
                    }
                }

                initLogEngine()
                // 使用Runtime执行命令
                if (TextUtils.isEmpty(packageName)) {
                    currentTaskProcess =
                        Runtime.getRuntime().exec("logcat -v time -f " + logFile?.absolutePath)
                    currentTaskProcess?.waitFor()
                } else {
                    currentTaskProcess = Runtime.getRuntime().exec("logcat -v time")
//                        获取输入流并转换为BufferedReader
                    val bufferedReader = BufferedReader(
                        InputStreamReader(currentTaskProcess!!.inputStream)
                    )
                    // 实时读取Logcat信息
                    var line: String?
                    while (bufferedReader.readLine().also { line = it } != null) {
                        // 输出Log信息
                        if (!TextUtils.isEmpty(packageName) && line?.contains(packageName) == true) {
                            call.invoke(line ?: "")
                            logFile?.appendText((line ?: "") + "\n")
                        } else if (TextUtils.isEmpty(packageName)) {
                            call.invoke(line ?: "")
                            logFile?.appendText((line ?: "") + "\n")
                        }
                    }
                }
            } catch (e: Throwable) {
                e.printStackTrace()
            }
        }
        logThread?.start()

        LogHttpHelper.service
            .deviceStatus(DeviceStatusReq(RemoteLogHelper.getLocalMac(), 1))
            .subscribeOn(RxUtils.getSchedulerIO())
            .subscribe({
                LocalLogUtils.log("上报一次状态成功：" + 1)
            }, {})
    }

    @Synchronized
    fun stopLogCatch(call: (logFile: File?) -> Unit) {
        if (currentTaskProcess == null) return
        currentTaskProcess?.destroy()
        currentTaskProcess = null
        logThread?.interrupt()
        logThread = null
        call.invoke(logFile)
        RemoteLogHelper.stopIntervalTask()
        currentStatus = LogState.TYPE_STOP
        //上传文件
        logFile?.apply {
            Observable.just(this).subscribeOn(RxUtils.getSchedulerIO()).subscribe({
                    try {
                        val zipFile = File(
                            RemoteLogHelper.getContext()?.externalCacheDir,
                            "log-$currentRandomId-${if (TextUtils.isEmpty(currentPackageName)) "all" else currentPackageName}" + ".zip"
                        )
                        ZipUtils.zipFiles(arrayOf(it), zipFile)
                        LogHttpHelper.service.getQiniuToken(
                            zipFile?.name ?: ""
                        ).retryWhen(RetryWithDelay()).subscribeOn(RxUtils.getSchedulerIO())
                            .subscribe({
                                CDNHelper.upload(it?.data?.token ?: "",
                                    it?.data?.key ?: "",
                                    zipFile,
                                    callback = { isSuccess, file ->
                                        LocalLogUtils.log("Upload file result : $file")
                                        if (isSuccess) {
                                            uploadLog(it?.data?.key ?: "", zipFile.length())
                                        }
                                    })
                                logFile?.deleteOnExit()
                            }, {
                                it.printStackTrace()
                            })
                    } catch (e: Exception) {
                        e.printStackTrace()
                    }
                }, {

                })
        }
    }

    fun getCurrentState(): LogState {
        if (logThread?.isAlive == true) {
            currentStatus = LogState.TYPE_CATCHING
            return currentStatus
        } else {
            return currentStatus
        }
    }
}