package com.tcl.remotelog.net

import android.app.Application
import retrofit2.Retrofit
import okhttp3.OkHttpClient
import com.tcl.remotelog.net.service.LogRequestService
import okhttp3.Cache
import retrofit2.converter.scalars.ScalarsConverterFactory
import retrofit2.converter.gson.GsonConverterFactory
import retrofit2.adapter.rxjava2.RxJava2CallAdapterFactory
import java.io.File
import java.lang.NullPointerException
import java.lang.RuntimeException
import java.util.*
import java.util.concurrent.TimeUnit

object LogHttpHelper  {
    private var mRetrofit: Retrofit? = null
    private var mMap: MutableMap<Class<*>, Any>? = null
    private val CONNECT_TIME_OUT = 30 //连接时间
    private val READ_TIME_OUT = 30 //读取时间
    private val WRITE_TIME_OUT = 30 //写入时间
    private var sContext: Application? = null
    private var sBaseURL: String? = null
    private var isInit = false

    /**
     * 缓存放在内部存储中，随应用卸载而删除，大小为10M。
     *
     * @return
     */
    private val cache: Cache
        private get() {
            val file =
                File(sContext!!.cacheDir, "response")
            return Cache(file, 20 * 1024 * 1024)
        }

    /**
     * 获取ApiService实例
     *
     * @param cls
     * @param <T>
     * @return
    </T> */
    fun <T> create(cls: Class<T>): T? {
        if (!mMap!!.containsKey(cls)) {
            val t = mRetrofit!!.create(cls)
            mMap!![cls] = t!!
        }
        return mMap!![cls] as T?
    }

    val service: LogRequestService
        get() = create(LogRequestService::class.java)!!

    fun init(context: Application?, baseURL: String?) {
        if (isInit) {
            return
        }
        if (context == null) {
            throw NullPointerException("context is null!")
        }
        sContext = context
        sBaseURL = baseURL
        isInit = true

        if (!isInit) {
            throw RuntimeException("please invoke init() at first!")
        }
        val builder = OkHttpClient.Builder()
            .connectTimeout(CONNECT_TIME_OUT.toLong(), TimeUnit.SECONDS)
            .readTimeout(READ_TIME_OUT.toLong(), TimeUnit.SECONDS)
            .writeTimeout(WRITE_TIME_OUT.toLong(), TimeUnit.SECONDS)
            .cache(cache)
        mRetrofit = Retrofit.Builder()
            .client(builder.build())
            .baseUrl(sBaseURL) //addConverterFactory解析有先后顺序，按需添加设置
            .addConverterFactory(ScalarsConverterFactory.create()) //转换为String对象
            .addConverterFactory(GsonConverterFactory.create()) //转换为Gson对象
            .addCallAdapterFactory(RxJava2CallAdapterFactory.create()) //rxJava集成
            .build()
        mMap = HashMap()
    }
}