package com.tcl.remotelog.utils

import android.app.Application
import android.util.Log
import com.qiniu.android.common.FixedZone
import com.qiniu.android.http.ResponseInfo
import com.qiniu.android.storage.*
import io.reactivex.Observable
import org.json.JSONObject
import java.io.File
import java.util.concurrent.CountDownLatch

/**
 * Copyright (C), 2015-2021, 编程猫有限公司
 * FileName: CDNHelper
 * Author: cretin
 * Date: 2021/3/30 9:46 AM
 * Description: cdn
 */
object CDNHelper {
    //本地存储一个
    private var uploadManager: UploadManager? = null

    /**
     * 初始化
     */
    fun init(application: Application) {
        val config: Configuration = Configuration.Builder().connectTimeout(90) // 链接超时。默认90秒
            .useHttps(true) // 是否使用https上传域名
            .useConcurrentResumeUpload(true) // 使用并发上传，使用并发上传时，除最后一块大小不定外，其余每个块大小固定为4M，
            .concurrentTaskCount(3) // 并发上传线程数量为3
            .responseTimeout(90) // 服务器响应超时。默认90秒
            .zone(FixedZone.zone2) // 设置区域，不指定会自动选择。指定不同区域的上传域名、备用域名、备用IP。
            .build()
        // 重用uploadManager。一般地，只需要创建一个uploadManager对象
        uploadManager = UploadManager(config)
    }

    /**
     * 上传单个文件
     */
    fun upload(
        token: String,
        key: String,
        file: File,
        callback: (isSuccess: Boolean, file: File?) -> Unit,
        progress: (persent: Int) -> Unit? = {}
    ) {
        Observable.just(file).map {
            val countDownLatch = CountDownLatch(0)
            var isSuccess = true
            uploadManager?.put(
                file, key, token, { key, info, response ->
                    countDownLatch.countDown()
                    if (info?.isOK() == true) {

                    } else {
                        isSuccess = false
                    }
                }, UploadOptions(
                    null, null, false, UpProgressHandler { key, percent ->
                        progress((percent * 100).toInt())
                    }, null
                )
            )
            try {
                countDownLatch.await()
            } catch (e: Throwable) {

            }
            val result = mutableMapOf<String, Any>()
            result["success"] = isSuccess
            result["datas"] = file
            result
        }.compose(RxUtils.applySchedulers()).subscribe({
            callback(it["success"].toString().toBoolean(), it["datas"] as File)
        }, {
            callback(false, null)
        })

    }

    /**
     * 批量上传
     */
    fun batchUpload(
        fileList: MutableMap<String, File>?,
        callback: (isSuccess: Boolean, fileList: MutableMap<String, File>?) -> Unit,
        progress: (persent: Int) -> Unit? = {}
    ) {
        if (fileList == null || fileList?.isEmpty() == true) return
        val allProgress = fileList.size * 100
        var currentProgress = 0.0

        fun updateProgress(map: MutableMap<String, Double>, call: () -> Unit) {
            var temp = 0.0
            map.forEach {
                temp += it.value
            }
            currentProgress = temp
            call.invoke()
        }

        Observable.just(fileList).map {
            val countDownLatch = CountDownLatch(fileList.size)
            var isSuccess = true
            val map = mutableMapOf<String, Double>()
            fileList.forEach {
                val split = it.key.split("#")
                val k = split[0]
                map.put(k, 0.0)
                uploadManager?.put(
                    it.value, split[0], split[1], object : UpCompletionHandler {
                        override fun complete(
                            key: String?, info: ResponseInfo?, response: JSONObject?
                        ) {
                            countDownLatch.countDown()
                            if (info?.isOK() == true) {

                            } else {
                                isSuccess = false
                            }
                        }
                    }, UploadOptions(
                        null, null, false, { key, percent ->
                            map.put(key, percent * 10000)
                            updateProgress(map) {
                                progress((currentProgress / allProgress).toInt())
                            }
                        }, null
                    )
                )
            }
            try {
                countDownLatch.await()
            } catch (e: Throwable) {

            }
            val result = mutableMapOf<String, Any>()
            result["success"] = isSuccess
            result["datas"] = fileList
            result
        }.compose(RxUtils.applySchedulers()).subscribe({
            callback(
                it["success"].toString().toBoolean(),
                it["datas"] as @kotlin.ParameterName(name = "fileList") MutableMap<String, File>
            )
        }, {
            callback(false, null)
        })
    }

    /**
     * 获取实际的url
     */
    fun getRealUrl(url: String): String {
        if (url.startsWith("http")) {
            return url
        }
        return url
    }
}