package com.tcl.remotelog.utils;

import android.util.Log;

import java.util.LinkedList;
import java.util.List;

public class RollingList<T> {
    private final LinkedList<T> dataList = new LinkedList<>();
    private int STEP_LENGTH_BIG = 2000;
    private int STEP_LENGTH_SMALL = 500;
    private FillListener<T> listener;

    private boolean isBigLog = false;

    public RollingList(FillListener<T> listener) {
        this.listener = listener;
    }

    public void setBigLogFlag(boolean isBigLog) {
        this.isBigLog = isBigLog;
    }

    public int getStepLength() {
        return isBigLog ? STEP_LENGTH_BIG : STEP_LENGTH_SMALL;
    }

    public void addData(T data) {
        synchronized (dataList) {
            dataList.addLast(data);
            if (!dataList.isEmpty() && dataList.size() % getStepLength() == 0) {
                if (listener != null) {
                    List<T> list = getHeaderList();
                    listener.fillStep(list);
                }
            }
        }
    }

    public LinkedList<T> getDataList() {
        return new LinkedList<>(dataList); // 返回副本，防止外部修改列表
    }


    public List<T> getHeaderList() {
        synchronized (dataList) {
            List<T> temp = new LinkedList<>();
            for (int i = 0; i < getStepLength() && !dataList.isEmpty(); i++) {
                temp.add(dataList.removeFirst());
            }
            return temp;
        }
    }

    public void reset() {
        if (listener != null) {
            listener.fillStep(dataList);
        }
        dataList.clear();
    }


    public interface FillListener<T> {
        void fillStep(List<T> datas);
    }
}
