package com.mxn.helper.logcat

import android.os.Bundle
import android.view.View
import android.widget.Button
import android.widget.EditText
import android.widget.TextView
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import com.mxn.helper.logcat.utils.NetworkUtils
import com.tcl.remotelog.RemoteLogHelper
import com.tcl.remotelog.log.LogState
import com.tcl.remotelog.utils.AppUtils.getAppVersion
import com.tcl.remotelog.utils.LocalLogUtils


class MainActivity : AppCompatActivity() {

    private var editText: EditText? = null
    private var btnStart: Button? = null
    private var btnStop: Button? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        supportActionBar?.hide()
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main_v2)

        editText = findViewById<EditText>(R.id.edit_filter)
        btnStart = findViewById<Button>(R.id.btn_start)
        btnStop = findViewById<Button>(R.id.btn_stop)
        editText?.requestFocus()

        findViewById<View>(R.id.tv_top).apply {
            postDelayed({
                this.visibility = View.GONE
            }, 2000)
        }

        if (RemoteLogHelper.getLogcatCurrentState() == LogState.TYPE_CATCHING) {
            selectStatus(false)
        } else {
            selectStatus(true)
        }

        initView()

        RemoteLogHelper.bindRefreshCall {
            LocalLogUtils.log("检查状态啦")
            editText?.postDelayed({
                selectStatus(true)
                setStatus()
            }, 2000)
        }

        findViewById<Button>(R.id.btn_start).setOnClickListener {
            RemoteLogHelper.getInitConfig(true, editText?.text?.toString() ?: "") { flag, msg ->
                runOnUiThread {
                    if (!flag) {
                        Toast.makeText(this, msg, Toast.LENGTH_SHORT).show()
                    } else {
                        Toast.makeText(this, "服务已启动，日志抓取中", Toast.LENGTH_SHORT).show()
                        selectStatus(false)
                        setStatus()
                    }
                }
            }
        }

        findViewById<Button>(R.id.btn_stop).setOnClickListener {
            RemoteLogHelper.stopLogCatch {
                runOnUiThread {
                    Toast.makeText(this, "日志已抓取并上传", Toast.LENGTH_SHORT).show()
                    setStatus()
                    selectStatus(true)
                }
            }
        }

        setStatus()
    }

    private fun selectStatus(flag: Boolean) {
        if (flag) {
            btnStop?.isEnabled = false
            btnStart?.isEnabled = true
            btnStart?.requestFocus()
        } else {
            btnStop?.isEnabled = true
            btnStart?.isEnabled = false
            btnStop?.requestFocus()
        }
    }

    private fun setStatus() {
        if (NetworkUtils.isNetworkAvailable(this)) {
            findViewById<TextView>(R.id.tv_info).text =
                "欢迎使用TCL远程日志工具 v${getAppVersion(this)}\n当前设备MAC地址："+NetworkUtils.getEthMacAddress(false) + "\n当前服务状态：" + getStatus()
        } else {
            Toast.makeText(this, "请连接网络", Toast.LENGTH_SHORT).show()
            findViewById<TextView>(R.id.tv_info).text =
                "欢迎使用TCL截图助手V1.0.0\n\n" + "请连接网络"
        }
    }

    private fun getStatus(): String {
        return if (RemoteLogHelper.getLogcatCurrentState() == LogState.TYPE_CATCHING) "日志抓取中" else "服务未启动"
    }

    override fun onBackPressed() {
        moveTaskToBack(true)
    }

    private fun initView() {

    }

}