package com.mxn.helper.logcat.utils;

import android.content.Context;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.net.Inet4Address;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

public class NetworkUtils {

    private static final String TAG = NetworkUtils.class.getSimpleName();
    public static final int TYPE_UNKNOWN = -1;
    public static final int TYPE_WIFI = 1;
    public static final int TYPE_MOBILE = 2;

    // 检查网络是否可用
    public static boolean isNetworkAvailable(Context context) {
        ConnectivityManager cm = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo activeNetwork = cm.getActiveNetworkInfo();
        return activeNetwork != null && activeNetwork.isConnectedOrConnecting();
    }

    // 获取网络类型
    public static int getNetworkType(Context context) {
        ConnectivityManager cm = (ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo activeNetwork = cm.getActiveNetworkInfo();
        if (activeNetwork != null && activeNetwork.isConnectedOrConnecting()) {
            int type = activeNetwork.getType();
            if (type == ConnectivityManager.TYPE_WIFI) {
                return TYPE_WIFI;
            } else if (type == ConnectivityManager.TYPE_MOBILE) {
                return TYPE_MOBILE;
            }
        }
        return TYPE_UNKNOWN;
    }

    public static String getEthMacAddress(boolean retainColon) {
        final String macPath = "/sys/class/net/eth0/address";
        File address = new File(macPath);
        String buff = null;
        if (address.exists()) {
            BufferedReader br;
            try {
                br = new BufferedReader(new FileReader(macPath));
                buff = br.readLine();
                if(!retainColon) {
                    buff =buff.replace(":", "");
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        if(buff!=null) {
            buff = buff.toUpperCase();
        }
        return buff;
    }

    // 获取IP地址
    public static List<String> getIPAddress(Context context) {
        List<String> ips = new ArrayList<>();
        String hostIp = null;
        try {
            for (Enumeration<NetworkInterface> en = NetworkInterface.getNetworkInterfaces(); en.hasMoreElements(); ) {
                NetworkInterface intf = en.nextElement();
                for (Enumeration<InetAddress> enumIpAddr = intf.getInetAddresses(); enumIpAddr.hasMoreElements(); ) {
                    InetAddress inetAddress = enumIpAddr.nextElement();
                    if (!inetAddress.isLoopbackAddress() && (inetAddress instanceof Inet4Address)) {
                        hostIp = inetAddress.getHostAddress().toString();
                    }
                }
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        ips.add(hostIp);
        return ips;

//        try {
//            Enumeration<NetworkInterface> networkInterfaces = NetworkInterface.getNetworkInterfaces();
//            while (networkInterfaces.hasMoreElements()) {
//                NetworkInterface networkInterface = networkInterfaces.nextElement();
//                Enumeration<InetAddress> inetAddresses = networkInterface.getInetAddresses();
//                while (inetAddresses.hasMoreElements()) {
//                    InetAddress inetAddress = inetAddresses.nextElement();
//                    if (!inetAddress.isLoopbackAddress() && !inetAddress.isLinkLocalAddress()
//                            && inetAddress.isSiteLocalAddress()) {
//                        System.out.println("IP Address: " + inetAddress.getHostAddress());
//                        ips.add(inetAddress.getHostAddress());
//                    }
//                }
//            }
//        } catch (SocketException e) {
//            e.printStackTrace();
//        }
//        return ips;
    }

    // 获取网络类型名称
    public static String getNetworkTypeName(Context context) {
        int type = getNetworkType(context);
        if (type == TYPE_WIFI) {
            return "WiFi";
        } else if (type == TYPE_MOBILE) {
            return "移动数据";
        } else {
            return "未知网络";
        }
    }
}
